#!/usr/bin/perl
use utf8;
use warnings;

use Net::IRC;
use POSIX qw(strftime);

use Data::Dumper;

my $usage;
$usage="
Run evalbot: ./evalbot.pl configfile

Where configfile is an evalbot config file (see config.sample)
";

$evalcmd = "./evalcmd.pl";

$conf = $ARGV[0];
if(!$conf) { die "$usage"; }
do "$conf" or die "Error when doing config file";

if(!$nick)     { die "nick unspecified"; }
if(!$password) { die "password unspecified"; }
if(!$server)   { die "irc server unspecified"; }
if(!$port)     { $port = 6667; }
if(!$owner)    { die "owner unspecified"; }
if($#channels == -1) { print "No channels specified. Continuing anyways\n"; }

my $msg_max = 300;


#print "connect to: @channels\n";

$SIG{'INT'} = 'my_sigint_catcher';
$SIG{'TERM'} = 'my_sigint_catcher';
$SIG{'QUIT'} = 'my_sigint_catcher';
$SIG{'ALRM'} = 'my_alarm';
sub my_sigint_catcher {
   exit(1);
}


# Set up the connection to the IRC server.
my $irc = new Net::IRC;
my $conn = $irc->newconn( Server => "$server",
    Port => "$port",
    Nick => "$nick",
    Ircname => "evalbot, owned by $owner" );


my $joined=0;

sub my_alarm {
    on_connect
}

sub on_connect {
    if($joined == 0) {
        print "connect..\n";
		if($nickserv) {
			$conn->sl("NICKSERV identify $nickserv");
		}
		foreach (@channels) {
			$conn->join( "$_" );
		}
        $joined=1;
    }

    $conn->privmsg("$nick", "ping");
    alarm(60);
}

sub on_user_input {
    #my ($ch, $msg) = <STDIN> =~ /^(.+?) *(.*)/;
    my ($ch, $msg) = split / /, <STDIN>, 2;

    chomp($msg);

    print ">>>>> $ch $msg\n";
    if ($ch eq "/eval") {
        eval $msg;
    } elsif ($ch =~ /^\/(reset)$/) {
        system('reset');
    } elsif ($ch =~ /^\/(q|quit)$/) {
        exit 0;
    } elsif ($ch =~ /^\/part$/) {
        $conn->part($msg);
    } elsif ($ch =~ /^\/(j|join)$/) {
        $conn->join($msg);
    } elsif ($ch =~ /^#/) {
        if ($joined) {
            $conn->privmsg($ch, $msg);
        }
    } else {
        print "help: #linux msg";
        return;
    }

}

sub on_kick {
    my( $conn, $event ) = @_;
    print Dumper(@_);
    if ($event->to eq $nick) {
        $conn->join($event->args->[0]);
    }
}

#alarm(60);



# Connect the handlers to the events.
$conn->add_handler( 376,      \&join_channel );
$conn->add_handler( 422,      \&join_channel );
$conn->add_handler( 'public', \&message );
$conn->add_handler( 'msg',    \&private );
$conn->add_handler( '376',    \&on_connect );
$conn->add_handler( 'kick',   \&on_kick );

#use IO::Select;
#$s = IO::Select->new();
#$s->add(\*STDIN);

#STDIN->blocking(0);
#$conn->addfh(\*STDIN, \&on_user_input, 'r' );
$irc->addfh(\*STDIN, \&on_user_input, 'r' );

# Start the Net::IRC event loop.
$irc->start;

sub join_channel
{
    my( $conn, $event ) = @_;

    print( "Currently online\n" );
}

sub message
{
    my( $conn, $event ) = @_;
    my( $msg ) = $event->args;

    # strip dengerous ctrl-a
    $msg =~ s/\x01//g;

    if( $msg =~/^\$ (.*)/ ) {

        print strftime("%Y-%m-%d %H:%M:%S> ", gmtime) . $event->nick."@".$event->to->[0]." run: $1\n";

        open(FOO, "-|", $evalcmd, $conf, $1);
        while(<FOO>) {
            print strftime("%Y-%m-%d %H:%M:%S> ", gmtime) . $event->nick ." out: $_";
            $conn->privmsg($event->to, $event->nick . ": $_");
        }
        close(FOO);
    }


}

sub private
{
     my( $conn, $event ) = @_;
     my( $msg ) = $event->args;

     if($event->nick =~ /^$nick$/) { return; } #lol

     # strip dengerous ctrl-a
     $msg =~ s/\x01//g;

     if($event->nick !~ /^$owner$/) {
         print "length: " . length $msg . "\n";
         for (my $offset = 0;length($msg) > $offset; $offset += $msg_max) {
             print "offset: $offset, max: $msg_max , substr: " . substr($msg, $offset, $msg_max) . "\n";
             $conn->privmsg( "$owner", "<". $event->nick. "> " . substr($msg, $offset, $msg_max));
         }
         # $conn->privmsg( "$owner", "< " . $event->nick . "> $msg" );
     }

     if($msg =~ /^!help.*/) {
         $conn->privmsg( $event->nick, "Usage:  \$ cmd" );
     } elsif($msg =~ /^!raw $password (.*)/) {
         $conn->sl($1);
     } elsif( $msg =~/^[\$\#]? ?(.*)/ ) {
         print strftime("%Y-%m-%d %H:%M:%S> ", gmtime) . $event->nick."@".$event->to->[0]." private run: $1\n";
         open(FOO, "-|", $evalcmd, $conf, "$1");
         while(<FOO>) {
             $conn->privmsg($event->nick, "$_");
         }
         close(FOO);
     }

}

#$VAR2 = bless( {
#                 'to' => [
#                           '_sh'
#                         ],
#                 'format' => 'kick',
#                 'from' => 'bt!~bt@osl-vps-10.ucd.ie',
#                 'user' => '~bt',
#                 'args' => [
#                             '#evologics',
#                             'flood'
#                           ],
#                 'nick' => 'bt',
#                 'type' => 'kick',
#                 'userhost' => '~bt@osl-vps-10.ucd.ie',
#                 'host' => 'osl-vps-10.ucd.ie'
#               }, 'Net::IRC::Event' );

